	page	64,132
;---------------------------------------------------------------:
;Title: vformat.asm                                             :
;Comment: the 5 routines in this module will link to            :
;	  the microsoft supplied format backbone module         :
;                                                               :
;Author				Vuong Tran                      :
;Date				11/14/84                        :
;Synopsis							:
;	Add 80 tracks/side as format option			:
;								:
;Date				12/04/84			:
;Synopsis							:
;	Make option /b to have only 8 sectors/track		:
;								:
;Date				12/10/84			:
;Synopsis							:
;	Use different offsets for new boot loader created on the:
;	same day						:
;								:
;Date				12/14/84			:
;Synopsis							:
;	Add table for double sided, double density, and 9	:
;	sectors/track						:
;	SET SECTORS/CLUSTER=2 FOR /H/8/1 OPTION                 :
;								:
;Date				12/18/84                        :
;Synopsis							:
;	No longer read boot sector. It is hardwired into 	:
;	<format.com>						:
;	Make variable <boot> to be public			:
;								:
;Date				01/04/85			:
;Synopsis							:
;	Change message for time out error to accomodate un-	:
;	allowed hard disk formatting.				:
;								:
;Date				01/08/85			:
;Synopsis							:
;	Change <offseto_density> from 119h to 1eh to accomodate	:
;	new boot loader dated 12/10/84				:
;								:
;Date				01/15/85			:
;Synopsis							:
;	Implements option /b like IBM				:
;	Forbids option /h/b					:
;---------------------------------------------------------------:


;segment must named CODE and classname 'CODE'

CODE	SEGMENT BYTE PUBLIC 'CODE'
	ASSUME	CS:CODE,DS:CODE,ES:CODE

;declare data PUBLIC

	PUBLIC	FATID, STARTSECTOR, SWITCHLIST, FREESPACE
	PUBLIC	FATSPACE, HARDFLAG, BOOT

;declare routines PUBLIC

	PUBLIC	INIT, DISKFORMAT, BADSECTOR, DONE, WRTFAT

;data defined in microsoft's format backbone

	EXTRN	SWITCHMAP:WORD, DRIVE:BYTE, SYSSIZ:WORD
		;switchmap: 0000_0000_0OH1_8BVS	;O is oldswitch
		;drive: 0 = A , 1 = B
	page

SW_HB		EQU	00100100B
SW_H18		EQU	00111000B
SW_H1		EQU	00110000B	;Bit position for switches h and 1
SW_H		EQU	00100000B	;bit position for switch h (80 tracks)
SW_B		EQU	00000100B	;bit position for switch b
sw_bs		equ	00000101b	;bit position for switch b and v
sw_8v		equ	00001010b	;bit position for switch 8 and v
sw_bv		equ	00000110b	;bit position for switch b and v
sw_1		equ	00010000b	;bit position for switch 1
sw_8		equ	00001000b	;bit position for switch 8
dos		equ	21h		;dos call
dos_pr_str	equ	09h		;dos print string command
rom_disk_io	equ	13h
rom_format	equ	05h		;rom disk format command
rom_write	equ	03h		;rom disk write command
rom_read	equ	02h		;rom disk read command
rom_verify	equ	04h		;rom disk verify command
rom_reset	equ	00h		;rom disk reset command
wrp_err		equ	03h		;rom disk write protect error
timeo_err	equ	80h		;rom disk motor time out error
CR	 	EQU	13
LF		EQU	10
GET_DEFAULT_DRIVE	EQU	19H
RESERVED_DOS_SIZE	EQU	1900H		;size of IBMDOS.COM
RESERVED_BIO_SIZE	EQU	0780H		;size of IBMBIO.COM
ROUNDUP_DOS_BIO		EQU	2200H		;round it up to a multiple of 256


	;* The following constants are fixed for a boot loader and
	;* should be changed for new versions of loader.

offseto_num_of_secclusters	equ	0DH	;offset to number of sectors/cluster
						;inside boot loader code
offseto_root_entries		equ	11H	;offset to max. number of root
						;directory entries inside boot loader
offseto_density			equ	1Eh	;offset to drive type inside boot loader

	;* Messages

sw_con_msg	db	'Parameters not compatible', CR, LF, '$'
time_out_msg	db	'Drive is not ready or drive is not floppy', CR, LF, '$'
wrt_pro_msg	db	'Attempt write-protect violation', CR, LF, '$'
tk0_msg		db	'Track 0 is Bad', CR, LF, '$'
wr_boot_err	db	'Unable to write boot', CR, LF, '$'
RD_BOOT_ERR	DB	'Unable to read boot', CR, LF, '$'
GO_MSG		DB	'Formatting ... ','$'
END_MSG		DB	'Format complete',CR,LF,'$'

MAX_TRACKS	DB	?	;maximum number of tracks/side
numsec		db	?	;number of sector/track
fatsec		db	?	;number of sectors used for fat area
track		db	?
side		db	?
retry		db	?
multsec		db	?

form_dat	db	?,?,1,2		;track #, head #, sector #, bytes/sector code
		db	?,?,2,2
		db	?,?,3,2
		db	?,?,4,2
		db	?,?,5,2
		db	?,?,6,2
		db	?,?,7,2
		db	?,?,8,2
		db	?,?,9,2

boot		db	512 dup (?)	;boot loader

	;Double sided, 9 sectors/track, 40 tracks
ldr_default	dw	112		;# of directory entries (word)
		dw	720		;# of sectors on disk (word)
		db	0fdh		;media descriptor (byte)
		dw	0002h		;# of sectors/fat (word)
		dw	0009h		;sectors/track (word)
		dw	0002h		;number of heads (word)

	;Single sided, 9 sectors/track, 40 tracks
ldr_1		dw	64		;# of directory entries (word)
		dw	360		;# of sectors on disk (word)
		db	0fch		;media descriptor (byte)
		dw	0002h		;# of sectors/fat (word)
		dw	0009h		;sectors/track (word)
		dw	0001h		;number of heads (word)

	;Double sided, 8 sectors/track, 40 tracks
ldr_8		dw	112		;# of directory entries (word)
		dw	640		;# of sectors on disk (word)
		db	0ffh		;media descriptor (byte)
		dw	0001h		;# of sectors/fat (word)
		dw	0008h		;sectors/track (word)
		dw	0002h		;number of heads (word)

	;Single sided, 8 sectors/track, 40 tracks
ldr_81		dw	64		;# of directory entries (word)
		dw	320		;# of sectors on disk (word)
		db	0feh		;media descriptor (byte)
		dw	0001h		;# of sectors/fat (word)
		dw	0008h		;sectors/track (word)
		dw	0001h		;number of heads (word)

	;Double-sided, 9 sectors/track, 80 tracks
ldr_h		dw	112		;# of directory entries (word)
		dw	1440		;# of sectors on disk (word)
		db	0f9h		;media descriptor (byte)
		dw	0003h		;# of sectors/fat (word)
		dw	0009h		;sectors/track (word)
		dw	0002h		;number of heads (word)

	;Single-sided, 9 sectors/track, 80 tracks (CONFLICT WITH HARDISK)
;ldr_h1		dw	112		;# of directory entries (word)
;		dw	720		;# of sectors on disk (word)
;		db	0f8h		;media descriptor (byte)
;		dw	0002h		;# of sectors/fat (word)
;		dw	0009h		;sectors/track (word)
;		dw	0001h		;number of heads (word)

	;Single-sided, 8 sectors/track, 80 tracks
ldr_h81		dw	112		;# of directory entries (word)
		dw	640		;# of sectors on disk (word)
		db	0fah		;media descriptor (byte)
		dw	0001h		;# of sectors/fat (word)
		dw	0008h		;sectors/track (word)
		dw	0001h		;number of heads (word)

	;Double-sided, 8 sectors/track, 80 tracks
ldr_h8		dw	112		;# of directory entries (word)
		dw	1280		;# of sectors on disk (word)
		db	0fbh		;media descriptor (byte)
		dw	0002h		;# of sectors/fat (word)
		dw	0008h		;sectors/track (word)
		dw	0002h		;number of heads (word)



FATID		db	?	;first id byte of fat
STARTSECTOR	dw	?	;the first data sector number
HARDFLAG	db	0
SWITCHLIST	db	6,'H18BVS'
FATSPACE	dw	fatbuf
FREESPACE	dw	endboot

DIR_ENTRIES	DW	?		;number of entries in directory
PATH_TO_IBMBIO	DB	'A:IBMBIO.COM',0
PATH_TO_IBMDOS	DB	'A:IBMDOS.COM',0
BUFFER_RW	DB	?

	page
INIT	proc	near
; init will check for switch conflicts

; entry: none
; exit: cbit set if any error

;check switch conflicts
;forbids option /h/b (01/15/85)


	PUSH	CS
	POP	ES
	mov	dx,SWITCHMAP		;get switch settings
 
	mov	al,dl
	and	al,sw_8v		;look at bit 3 (/8) and bit 1 (/V)
	cmp	al,sw_8v		;are they both set
	je	sw_conflict		;yes,then switch conflict

	mov	al,dl
	and	al,sw_bv		;look at bit 2 (/B) and bit 1 (/V)
	cmp	al,sw_bv		;are they both set
	je	sw_conflict		;yes,then switch conflict

	mov	al,dl
	and	al,sw_hb		;look at bit 2 (/B) and bit 5 (/h)
	cmp	al,sw_hb		;are they both set
	je	sw_conflict		;yes,then switch conflict

	mov	al,dl
	and	al,sw_bs		;check for /b/s switch conflict
	cmp	al,sw_bs		;	/
	je	sw_conflict		;	/

	mov	al,dl
	and	al,SW_H18		;check for /H/1 switch conflict
	cmp	al,SW_H1		;	/
	je	sw_conflict		;	/

	MOV	AL, DL
	TEST	AL, SW_B                ;is it switch /b?
	JZ	NOT_SW_B
	AND	AL, 0FEH		;turn off system switch (/S)
	OR	AL, SW_8		;turn on switch 8 (/8)
	MOV	BYTE PTR SWITCHMAP, AL

NOT_SW_B:
	;* NO OP PART OF CODE THAT READS BOOT SECTOR
;	MOV	AH, GET_DEFAULT_DRIVE	;get default drive in (al)
;	INT	21H			;	/
;	LEA	BX, BOOT                ;ES:BX points to memory
;	MOV	CH, 0			;trach #0
;	MOV	CL, 1			;sector #1
;	MOV	DH, 0			;side 0
;	MOV	DL, AL			;source drive
;	MOV	AL, 01H			;# of sectors=1
;	CALL	rd_sec			;read boot sector into memory
;	JNC	READ_BOOT_DONE          ;go if no read error
;	LEA	DX, RD_BOOT_ERR         ;else display error message
;	CALL	PR_STR                  ;	/
	RET


sw_conflict:

	lea	dx,sw_con_msg
	call	pr_str			;print error message

READ_BOOT_DONE:
	ret
INIT	endp
	page
DISKFORMAT	proc	near
; diskformat will set various format related parameters for later use
; diskformat will patch the boot loader in memory with the proper parameters
; diskformat will format track 0 of target drive (both sides if not /1)
; diskformat will write the boot loader to boot sector of target drive

; entry: none
; exit: return error with cbit set if format error or disk write error
;	clear cbit if no error

;set parameters according to  /h, /1 and /8 switch settings

	LEA	DX, GO_MSG
	CALL	PR_STR

	push	cs			;es has the same value as cs
	pop	es

	mov	bx,offset boot + offseto_num_of_secclusters 	;bx points to sectors/cluster of boot loader
	MOV	MAX_TRACKS, 80					;maximum number of tracks/side

	TEST	BYTE PTR SWITCHMAP, SW_H	;80 TRACKS?
	JNZ	TRACKS_80                       ;go if formatting for 80 tracks
	JMP	TRACKS_40
TRACKS_80:
	mov	si, offset boot + offseto_density 	;set quad_density mode
	mov	byte ptr [si], 40h                      ;
	TEST	BYTE PTR SWITCHMAP, SW_8
	JNZ	SET_H8
;	TEST	BYTE PTR SWITCHMAP, SW_1
;	JNZ	SET_H91

set_h9:   				;80 tracks, 9 sectors/track, double-sided
	MOV	FATSEC, 6		;fat sectors = 6
	MOV	NUMSEC, 9		;sectors/track
	MOV	FATID, 0f9h		;fat ID = 0F9H
	MOV	STARTSECTOR, 15		;1st data sector
	;------ patch loader for /h or /h/9
	MOV	BYTE PTR [BX], 2	;sectors/cluster = 2
	LEA	SI, LDR_H
	JMP	PATCH_LDR

;set_h91: 				;80 tracks, 9 sectors/track, single-sided
;	MOV	FATSEC, 4		;fat sectors = 4
;	MOV	NUMSEC, 9		;sectors/track
;	MOV	FATID, 0F8h		;fat ID = 0F8H (NOTE: Hardisk has the same
					;		media byte)
;	MOV	STARTSECTOR, 13		;1st data sector
	;------ patch loader for /h/1 or /h/9/1
;	MOV	BYTE PTR [BX], 2	;sectors/cluster = 2
;	LEA	SI, LDR_H1
;	JMP	PATCH_LDR

set_h8:	TEST	BYTE PTR SWITCHMAP, SW_1	;80 tracks, 8 sectors/track, double-sided
	JNZ	set_h81
	MOV	FATSEC, 4		;fat sectors = 4
	MOV	NUMSEC, 8		;sectors/track
	MOV	FATID, 0FBH		;fat ID = 0FBH
	MOV	STARTSECTOR, 13		;1st data sector
	;------ patch loader for /h/8
	MOV	BYTE PTR [BX], 2	;sectors/cluster = 2
	LEA	SI, LDR_H8
	JMP	PATCH_LDR

set_h81:				;80 tracks, 8 sectors/track, single-sided
	MOV	FATSEC, 2		;fat sectors = 2
	MOV	NUMSEC, 8		;sectors/track
	MOV	FATID, 0FAH		;fat ID = 0FAH
	MOV	STARTSECTOR, 11		;1st data sector
	;------ patch loader for /h/8/1
	MOV	BYTE PTR [BX], 2	;sectors/cluster = 2
	LEA	SI, LDR_H81
	JMP	PATCH_LDR

TRACKS_40:
	MOV	MAX_TRACKS, 40		;maximum number of tracks/side
	mov	si, offset boot + offseto_density 	;set double density mode
	mov	byte ptr [si], 0h                       ;
	test	byte ptr SWITCHMAP,sw_8	;is /8 switch set
	jnz	set8			;yes, set parameter for 8 sector format
					;else, set parameter for 9 sector format
set9:	mov	fatsec,04h		;fat sectors = 4
	mov	numsec,09h		;sectors/track = 9
	
	test	byte ptr SWITCHMAP,sw_1	;is /1 switch set
	jnz	set91			;yes, set parameter for 1 sided
					;else, set for 2 sided
	mov	FATID,0fdh		;fat id = 0fdh
	mov	STARTSECTOR,000dh	;first data sector is 13

	;patch loader for double-density, double-sided, 9 sectors/track
	mov	byte ptr [bx], 2 ;sectors/cluster =2
	lea	si, ldr_default
	jmp	short patch_ldr

set91:	mov	FATID,0fch		;fat id = 0fch
	mov	STARTSECTOR,000ah	;first data sector is 10

	;patch loader for /1 only

	mov	byte ptr [bx],1		;sectors/cluster = 1
	lea	si, ldr_1 		;si points to ldr_1 for block move
	jmp	short patch_ldr		;all set, goto patch boot loader

set8:	mov	fatsec,02h		;fat sectors = 2
	mov	numsec,08h		;sectors/track = 8
	
	test	byte ptr SWITCHMAP,sw_1	;is /1 switch set
	jnz	set81			;yes, set parameter for 1 sided
					;else, set for 2 sided
	mov	FATID,0ffh		;fat id = 0ffh
	mov	STARTSECTOR,000bh	;first data sector is 11

	;patch loader for /8 only

	mov	byte ptr [bx],2		;sector/cluster = 2
	lea	si, ldr_8		;si points to ldr_8 for block move
	jmp	short patch_ldr		;all set, goto patch boot loader

set81:	mov	FATID,0feh		;fat id = 0feh
	mov	STARTSECTOR,0008h	;first data sector is 8

	;patch loader for /8 and /1

	mov	byte ptr [bx],1		;sector/cluster = 1
	lea	si, ldr_81		;si points to ldr_81 for block move

patch_ldr:	;patch boot loader

	mov	cx,11			;counter = 11 for block move
	lea	di, boot + offseto_root_entries		;patch starts from here
	rep	movsb			;do block byte move

form_tk0:	;format track 0

	mov	ah,0			;reset drive
	int	rom_disk_io

	mov	track,0			;track = 0
	mov	side,0			;side = 0
	call	form_ver		;format and verify the track

	jc	tk0_err			;if error, goto print message

	test	byte ptr SWITCHMAP,sw_1	;see if /1 is selected
	jnz	tranboot		;yes, then skip format track 0 side 1

	mov	side,1			;side = 1
	call	form_ver		;format and verify the track

	jnc	tranboot		;if no error, then tracnsfer boot loader

tk0_err:	;print track 0 error and return

	lea	dx,tk0_msg
	call	pr_str			;print error message
	ret

tranboot:	;transfer boot loader from memory to boot sector
 
	lea	bx,boot			;es:bx points to boot loader
	mov	ch,0			;ch = track = 0
	mov	cl,1			;cl = sector # = 1
	mov	dh,0			;dh = side = 0
	mov	dl,DRIVE		;dl = drive
	mov	al,1			;al = # of sector = 1
	call	wr_sec			;write the sector

	jnc	trandone		;if no write error, done

	lea	dx,wr_boot_err
	call	pr_str			;print error message

trandone:
	ret
DISKFORMAT	endp
	page
BADSECTOR	proc	near
; badsector will format the target disk from track 1 to 39 if mode 40
;Else, it will format from track 1 to 79 for mode 80
; badsector will report badsector number to format

; entry: none
; exit: cbit set if hard error (hard error will cause format to abort)
;	cbit cleared if soft error (soft error means reporting of badsector,
;				    format will renter BADSECTOR afterwards)
;	To report badsectors:
;		bx contains the badsector sector number 
;		ax contains the number of consecutive badsectors
;	For final return to format:
;		ax = 0 ; bx = don't care ; cbit = cleared

	push	cs
	pop	es

;check for rentry

	cmp	track,0			;if track=0 then first entry to this routine
	jnz	rentry			;if not first time, skip to rentry
	
	mov	ah,rom_reset		;else, reset drive
	int	rom_disk_io

	mov	track,01		;make track = 1 to indicate rentry for next time
	mov	side,0			;make side 0 flag 0 to indicate side 0 is formatted
	 
rentry:		;while track < 40 , format tracks

	cmp	side,0			;see if side 0 of last entry is formatted
	jnz	form_side0		;if not, format it this time

	push	ax
	mov	al, track
	cmp	al, max_tracks		;if track >= <max_tracks>
 pop	ax
	jae	alldone			;then all done

	;determine format 1 side or 2 sides

	test	byte ptr SWITCHMAP,sw_1	;look at bit 4 (/1)
	jnz	form_side0		;if set, format side 0 only

form_side1:	;format and verify track on side 1

	mov	side,1			;side = 1
	call	form_ver		;format and verify the track

	jc	report			;if format error, report it

form_side0:	;format and verify track on side 0

	mov	side,0			;side = 0
	call	form_ver		;format and verify the track

	inc	track			;increament current track number

	jnc	rentry			;if no error, format next track

	;if format or verify error, report badsector number in bx
	;and consecutive badsectors in ax
report:					
	mov	al,track		;al = track number
	test	byte ptr SWITCHMAP,sw_1	;is /1 selected
	jnz	rp1			;yes, then skip multiply

	add	al,al			;al = al * 2 (al = track * 2)
rp1:	cmp	side,0			;is it from side 1 or side 0
	jz	rp2			;from side 0, skip increament

	inc	al			;al = al + 1 (al = track * 2 + 1)
rp2:	mul	numsec			;ax = al * numsec
	mov	bx,ax			;bx has the value
	mov	al,numsec		;ax = number of consecutive badsector
	cbw				;clear ah
	clc
	ret

alldone:	;final return to format, ax = 0 for indication

	xor	ax,ax
	clc
	ret
BADSECTOR	endp
	page
WRTFAT	proc	near
; write fat space out to the target drive track 0

; entry: none
; exit: cbit set if error in write
;	cbit cleared if no error

	push	cs
	pop	es

	MOV	BX, WORD PTR FATSPACE	;es:bx points to memory
	MOV	SI, BX			;build 2nd FAT in memory
	MOV	AL, BYTE PTR FATSEC     ;
	CBW				;
	MOV	DI, AX			;
	MOV	CL, 08			;
	SHL	DI, CL			;
	MOV	CX, DI			;
	ADD	DI, SI                  ;
REP	MOVSB                           ;

	mov	al,fatsec		;al = multiple sector count = fatsec
	mov	cl,2			;fat starts from sector 2
	mov	ch,0			;track 0
	mov	dl,DRIVE		;target drive
	mov	dh,0			;side 0

	call	wr_sec			;write the fat sectors

	ret
WRTFAT	endp
	page

DONE	proc	near
;Process /B option like IBM

		MOV	DX, OFFSET END_MSG
		CALL	PR_STR
		TEST	WORD PTR SWITCHMAP, SW_B
		JNZ	PROCESS_B
		JMP	QUAD_DENSITY
PROCESS_B:	MOV	AL, [DRIVE]	;get drive
		ADD	AL, 41H
		MOV	BYTE PTR PATH_TO_IBMBIO, AL
		MOV	BYTE PTR PATH_TO_IBMDOS, AL

	;Create a dummy IBMBIO.COM
		MOV	CX, 06		;make file hidden and system
		MOV	DX, OFFSET PATH_TO_IBMBIO
		MOV	AH, 3CH		;create IBMBIO.COM file
		INT	21H		;
		MOV	BX, AX		;get IBMBIO.COM file handle
		MOV	CX, RESERVED_BIO_SIZE	;set size of file in bytes
		MOV	DX, OFFSET BUFFER_RW	;points to garbage
		MOV	AH, 40H		;write to a file
		INT	21H		;
		MOV	AH, 3EH		;close file IBMBIO.COM
		INT	21H		;

	;Create a dummy IBMDOS.COM
		MOV	CX, 06		;make file hidden and system
		MOV	DX, OFFSET PATH_TO_IBMDOS
		MOV	AH, 3CH		;create IBMDOS.COM file
		INT	21H		;
		MOV	BX, AX		;get IBMDOS.COM file handle
		MOV	CX, RESERVED_DOS_SIZE	;set size of file in bytes
		MOV	DX, OFFSET BUFFER_RW	;points to garbage
		MOV	AH, 40H		;write to a file
		INT	21H		;
		MOV	AH, 3EH		;close file IBMDOS.COM
		INT	21H		;
		ADD	WORD PTR SYSSIZ, ROUNDUP_DOS_BIO
QUAD_DENSITY:
		TEST	WORD PTR SWITCHMAP, SW_H
		JZ	CHECK_8    		;go if double density
		JMP	FORMAT_DONE
CHECK_8:	TEST	WORD PTR SWITCHMAP, SW_8
		JNZ	FIX_DIR    		;go if 8 sectors/track option
                JMP	FORMAT_DONE

	;Write 0E5H to the 1st byte of all directory entries
	;This will make the disk compatible with IBM DOS 1.XX

FIX_DIR: 	MOV	WORD PTR DIR_ENTRIES, 112	;for /8
		TEST	WORD PTR SWITCHMAP, SW_1
		JZ	FIX_DIR0                ;go if double sided
		MOV	WORD PTR DIR_ENTRIES, 64	;FOR /8/1
FIX_DIR0:	MOV	AL, [DRIVE]
		CBW
		MOV	CX, 07H			;7 sectors to read
 		MOV	DX, 03H			;logical sector = 3 (directory)
		MOV	BX, WORD PTR FATSPACE	;DS:BX = xfer address for dir sectors
		INT	25H			;absolute directory read
		POP	AX			;
		JNB	FIX_DIR1                ;go if no read error
ERROR:		RET
FIX_DIR1:	MOV	BX, WORD PTR FATSPACE
		MOV	BYTE PTR [BX+0BH], 6	;make files hidden and system
		MOV	BYTE PTR [BX+2BH], 6	;
		MOV	AX, WORD PTR DIR_ENTRIES
		DEC	AX
		MOV	BX, 20H			;size of dir entry
		MUL	BX
		ADD	AX, WORD PTR FATSPACE	;AX --> last dir entry
		MOV	BX, AX
		XOR	AX, AX
		MOV	AH, 0E5H
FIX_DIR3:	CMP	BYTE PTR [BX], AL       ;is 1st byte of last dir entry = 0?
		JNZ	FIX_DIR2                ;go if no
		MOV	BYTE PTR [BX], AH	;write out 0E5H
		SUB	BX, 20H			;move backwards to previous dir entry
		CMP	BX, WORD PTR FATSPACE	;out of entries?
		JNB	FIX_DIR3
FIX_DIR2:	MOV	AL, [DRIVE]             ;update dir entries on target drive
		CBW                             ;
		MOV	CX, 7                   ;
		MOV	DX, 3                   ;
		MOV	BX, WORD PTR FATSPACE   ;
		INT	26H 			;
		POP	AX			;
		JB	ERROR

FORMAT_DONE:	CLC
		RET
DONE	endp
	page
form_ver	proc	near
;format and verify the specified track 

;entry: track = the track to format and verify
;	side = the side of the track (0 or 1)
;exit:	cbit set if any error
;	cbit cleared if no error

	mov	ah,track		;ah = current track for format data
	mov	al,side			;al = side to format = 1 for format data
	call	init_form_dat		;initialize format data

	call	form_trk		;format the track

	jc	ret_fv			;if format error, ret

	call	ver_trk			;verify the track
					;ret despite of error or not
ret_fv:	ret
form_ver	endp
	page
init_form_dat	proc	near
; initialize format data for use in rom format

; entry: ah = track number , al = side number
; exit: none

	lea	bx,form_dat		;es:bx points to format data
	mov	cx,09			;init 9 blocks for 9 sectors

init_next:
	mov	byte ptr [bx],ah	;track information
	mov	byte ptr [bx+1],al	;side information
	add	bx,0004h		;point to next block
	loop	init_next

	ret
init_form_dat	endp
	page
form_trk	proc	near
; format track with retries and error handling

; entry: set the side to format in DH
;	 format data initialized
; exit: cbit set if error
;	cbit cleared if no error

;set format informations for the rom

	lea	bx,form_dat		;es:bx points to format data
	mov	dh,side			;dh = side to format
	mov	dl,DRIVE		;dl = target drive
	TEST	BYTE PTR SWITCHMAP, SW_H	;80 TRACKS MODE?
	JZ	FORM_MODE40			;go if 40 tracks mode
	OR	DL, 40H				;set 80 tracks mode
FORM_MODE40:
	mov	ch,track		;ch = track to format
	mov	cl,01			;cl = sector number = 1
	mov	retry,4			;retry count = 4

;call rom to format the track

form_next:
	mov	al,numsec		;al = sector/track
	mov	ah,rom_format
	int	rom_disk_io
	jnc	form_ret		;if no format error, good return

form_err:
	test	ah,timeo_err or wrp_err	;check if timeout or wrt protect err
	jz	form_reset
	jmp	err_proc		;yes then print error msg

form_reset:
	mov	ah,0			;no, then reset drive and retry
	int	rom_disk_io

	dec	retry			;dec retry count
	jnz	form_next		;retry if retry count not 0

	stc
form_ret:
	ret
form_trk	endp
	page
ver_trk	proc	near
; verify track with retries and error handling

; entry: set the side to verify in DH
; exit: cbit set if error

;set verify informations for the rom

	mov	dh,side			;dh = side to format
	mov	dl,DRIVE		;dl = target drive
	TEST	BYTE PTR SWITCHMAP, SW_H;80 tracks mode?
	JZ	VER_MODE40              ;go if 40 tracks mode
	OR	DL, 40H			;set 80 tracks mode
VER_MODE40:
	mov	ch,track		;ch = track to format
	mov	cl,01			;cl = sector number = 1
	mov	retry,3			;only 3 retries

;call rom to verify track

ver_next:
	mov	al,numsec		;al = sector/track
	mov	ah,rom_verify
	int	rom_disk_io
	jc	ver_err			;if verify error

	ret				;good return

ver_err:
	test	ah,timeo_err or wrp_err ;check if timeout or wrt protect err
	jnz	err_proc		;if yes, print error msg

	mov	ah,0			;reset drive
	int	rom_disk_io

	dec	retry			;dec retry count
	jnz	ver_next		;retry if retry count not 0

	stc				;error return
	ret
ver_trk	endp
	page
wr_sec	proc	near
; write sectors to the disk with retries and error handling

; entry: es:bx points to the memory buffer
;	 al = number of sectors to write
;	 ch = track to write on
;	 cl = sector to write on
;	 dh = side to write on
;	 dl = drive to write to
; exit:	cbit set if write error

	TEST	BYTE PTR SWITCHMAP, SW_H	;80 TRACKS MODE?
	JZ	WR_MODE40			;GO IF 40 TRACKS MODE
	OR	DL, 40H				;SET 80 TRACKS MODE
WR_MODE40:
	mov	retry,4			;retry = 4
	mov	multsec,al		;save number of sector to write
wr_next:
	mov	al,multsec
	mov	ah,rom_write		;call rom to write
	int	rom_disk_io
	jc	wr_err			;if write error

	ret				;good return

wr_err:
	test	ah,timeo_err or wrp_err	;check if timeout or wrt protect err
	jz	wr_retry		;no, then retry

	call	err_proc		;yes then print err msg (no return here)

wr_retry:
	mov	ah,0			;else reset drive
	int	rom_disk_io

	dec	retry			;dec retry count
	jnz	wr_next			;if retry not zero

	stc				;else bad return
	ret
wr_sec	endp
	page

rd_sec	proc	near
; read sectors from the disk with retries and error handling

; entry: es:bx points to the memory buffer
;	 al = number of sectors to read
;	 ch = track to read from
;	 cl = sector to read from
;	 dh = side to read from
;	 dl = drive to read from
; exit:	cbit set if read error

	mov	retry,4			;retry = 4
	mov	multsec,al		;save number of sector to read
rd_next:
	mov	al,multsec
	mov	ah,rom_read		;call rom to read
	int	rom_disk_io
	jc	rd_retry		;if read error

	ret				;good return

rd_retry:
	mov	ah,0			;else reset drive
	int	rom_disk_io

	dec	retry			;dec retry count
	jnz	rd_next			;if retry not zero

	stc				;else bad return
	ret
rd_sec	endp
	page

err_proc	proc	near
; err_proc will print out the approiate error message and return with cbit set
; err_proc is a black hole , it will not return to the caller but instead to 
; the caller's caller's caller.

;entry: ah = disk error condition 
;exit: cbit set

	test	ah,80h			;timeout error?
	jz	ep1			;no, then check next

	lea	dx,time_out_msg
	call	pr_str			;print error message
	jmp	short ep2

ep1:	test	ah,03h			;write protect violation?
	jz	ep2			;no, then check next

	lea	dx,wrt_pro_msg
	call	pr_str			;print error message

;	jmp	ep2

ep2:	;no more for now

	pop	ax			;trash caller's return addr
	pop	ax			;trash caller's caller's return addr
	stc
	ret				;error return to FORMAT
err_proc	endp
	page
pr_str	proc	near
; call dos to print the string, set carry and return

; entry: dx has the effective address of the string
; exit: cbit always set

	mov	ah,dos_pr_str
	int	dos

	stc
	ret
pr_str	endp



fatbuf		db	7 * 512 dup(?) ;this space is used either to
					;save FAT or DIRECTORY sectors
endboot		label	byte

CODE	ENDS
	END
